#include <DHT.h>
#include <LedControl.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

#define DHTTYPE DHT11
#define DHTPIN 13

#define PHOTORESISTOR_PIN A0
#define SOILHUM_PIN A3

byte PRINTED_LCD = 1;
byte PRINTED_MATRIX = 1;

#define GOOD_TEMP 15;
#define GOOD_HUMIDITY 50;
#define GOOD_LIGHT 50;
#define GOOD_SOIL_HUM 50;

#define DIN 12;
#define CS 11;
#define CLK 10;

int half_seconds = 0;
#define NOTIFY_INTERVAL = 7200;               // timer counts by 0.5 seconds, so 1 hour corresponds to 3600 * 2 = 7200

byte happy[8] = {0x00, 0x08, 0x24, 0x02, 0x02, 0x24, 0x08, 0x00,};
byte sad[8] = {0x00, 0x02, 0x24, 0x08, 0x08, 0x24, 0x02, 0x00,};
boolean sad_plant = false;

LiquidCrystal_I2C lcd(0x27, 16, 2);          // set the LCD address to 0x27 for a 16 chars and 2 line display
LedControl lc = LedControl(DIN, CLK, CS, 0);

DHT dht(DHTPIN, DHTTYPE);

struct params {
  float temp;
  int hum;
  int light;
  int soil_hum;
};
typedef struct params PARAMS;

PARAMS p = {0, 0, 0, 0};
PARAMS good_params  = {GOOD_TEMP, GOOD_HUMIDITY, GOOD_LIGHT, GOOD_SOIL_HUM};


void setup_TIMER()
{
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1  = 0;

  // Frequency 1Hz
  TCCR1B |= (1 << WGM12);    // CTC mode
  TCCR1B |= (1 << CS12);     // 256 prescaler
  OCR1A = 31249;             // TOP value

  TIMSK1 |= (1 << OCIE1A);   // unmask CTC OCCR1A
}

void setup_LCD()
{
  lcd.init();
}

void setup_MATRIX()
{
  lc.shutdown(0, false);     // the MAX72XX is in power-saving mode on startup
  lc.setIntensity(0, 5);     // set the brightness
  lc.clearDisplay(0);        // clear the display
}

void setup() {
  
  pinMode(DHTPIN, INPUT);
  pinMode(PHOTORESISTOR_PIN, INPUT);
  pinMode(SOILHUM_PIN, INPUT);

  dht.begin();
  
  Serial.begin(9600);
  
  setup_TIMER();
  setup_LCD();
  setup_MATRIX();

  sei();                    // activate global intrerrupt
}

ISR(TIMER1_COMPA_vect)
{
  half_seconds++;
}

void display_LCD() {

  if (!PRINTED_LCD) {
    lcd.clear();
    lcd.backlight();

    lcd.setCursor(0, 0);
    lcd.print("Temp: ");
    lcd.print(p.temp);
    lcd.print(" *C ");
    
    lcd.setCursor(0, 1);
    lcd.print("Soil:");
    lcd.print(p.soil_hum);
    lcd.print("%");

    lcd.print("Light:");
    lcd.print(p.light);
    lcd.print("%");

    PRINTED_LCD = 1;
  }
}

void printByte(byte character []) {
  int i = 0;
  for (i = 0; i < 8; i++) {
    lc.setRow(0, i, character[i]);
  }
}

void displayMatrix() {
  if (!PRINTED_MATRIX) {
    lc.clearDisplay(0);
    if(sad_plant) printByte(sad);
    if(!sad_plant) printByte(happy); 
    PRINTED_MATRIX = 1;
  }
}

void readDHT11() {
    float h = dht.readHumidity();
    float t = dht.readTemperature();
    
    if (isnan(h) || isnan(t)) {                              // check if any reads failed and exit
      Serial.println("Failed to read from DHT sensor!");
      return;
    }

    float hic = dht.computeHeatIndex(t, h, false);

    p.hum = h;
    p.temp  = hic;  
}

void readPhotoresistor() {
    int value = analogRead(PHOTORESISTOR_PIN);              // read analog input
    int lightProc = map(value, 1023, 0, 0, 100);            // transform to percent

    p.light = lightProc;
}

void readSoilHum() {
    int value = analogRead(SOILHUM_PIN);                    // read analog input
    int proc = map(value, 1023, 0, 0, 100);                 // transform to percent

    p.soil_hum = proc;
}

void verify_params_and_notify()
{
  cli();
  
  if (half_seconds >= NOTIFY_INTERVAL) {                    // 1 hour passed, read values from sensors 
     PRINTED_LCD = 0;
     PRINTED_MATRIX = 0;
     
     readDHT11();
     readPhotoresistor();
     readSoilHum();
    
     String low_params = "";
     sad_plant = false;
     
     if (p.light < good_params.light) {
        low_params += "light";
        sad_plant = true;
     }

     if (p.temp < good_params.temp) {
        low_params += ",";
        low_params += "temp";
        sad_plant = true;
     }

     if (p.hum < good_params.hum) {
        low_params += ",";
        low_params += "hum";
        sad_plant = true;
     }

      if (p.soil_hum < good_params.soil_hum) {
        low_params += ",";
        low_params += "soil";
        sad_plant = true;
     }
     

     if (sad_plant) Serial.println(low_params);               // send a message to the ESP to build a notification
     
     seconds = 0;
  }

  sei();
}

void loop()
{
  verify_params_and_notify();
  display_LCD();
  displayMatrix();
}
